#include "config.h"

#include "gettext.h"
#include "playlist.h"
#include "music.h"
#include "main.h"
#include "parsefile.h"

#include <string.h>
#include <sys/types.h> /* for stat() */
#include <sys/stat.h>  /* for stat() */
#include <unistd.h>    /* for stat() */
#include <errno.h>

GList *playlist = NULL;
GList *playlist_current = NULL;

static GList *playlist_random = NULL;

void
playlist_init ()
{
}

void
playlist_destroy ()
{
    playlist_clear ();
}

static struct PlaylistItem*
listitem_new(gchar *path, gchar **p, guint nalts)
{
    struct PlaylistItem *s;

    s = g_new (struct PlaylistItem, 1);
    s->main_path = path;
    s->paths = p;
    s->use_path = 0; /*they cycle to the end.. g_random_int_range (0, nalts);*/
    return s;
}

static void
listitem_free (struct PlaylistItem *item)
{
    g_strfreev (item->paths);
    g_free (item->main_path);
    g_free (item);
}

/*
static gboolean
get_file_utf8 (const gchar *path, gchar **f, gchar **u)
{
    if (g_utf8_validate (path, -1, NULL)) {
        if (!(*f = g_filename_from_utf8 (path, -1, NULL, NULL, NULL))) {
            g_critical (_("Skipping '%s'. Could not convert from UTF-8. "
                          "Bug?"), path);
            return FALSE;
        }
        *u = g_strdup (path);
    } else {
        if (!(*u = g_filename_to_utf8 (path, -1, NULL, NULL, NULL))) {
            g_warning (_("Skipping '%s'. Could not convert to UTF-8. "
                         "See the README for a possible solution."), path);
            return FALSE;
        }
        *f = g_strdup (path);
    }
    return TRUE;
}
*/

static void
playlist_append (PlaylistItem *item)
{
    gint p;

    music_notify ("appending %s", MAIN_PATH (item));
    playlist = g_list_append (playlist, item);

    if (!playlist_current) playlist_current = playlist;

    p = g_list_position (playlist_random,
                         g_list_find (playlist_random,
                                      playlist_current->data));
    playlist_random = g_list_insert
        (playlist_random, item, 
         g_random_int_range (p, g_list_length (playlist)) + 1);
}

void
playlist_append_single (const gchar *path)
{
    gchar **paths;

    g_return_if_fail (path != NULL);
    g_return_if_fail (g_utf8_validate (path, -1, NULL));

    if (!parse_file (path)) /* XXX PASS A GERROR**? */
        return;

    paths = g_new (gchar*, 2);
    paths[0] = g_strdup (path);
    paths[1] = NULL;

    playlist_append (listitem_new (g_strdup (path), paths, 1));
}

void
playlist_append_alternatives (const gchar *path, gchar *const* alts)
{
    gchar **paths;
    guint i, nalts = 0;

    g_return_if_fail (alts != NULL && alts[0] != NULL);

    /* XXX errors encountered here should go thru the client fifos */

    /* count 'em */
    while (alts[nalts]) ++nalts;

    paths = g_new (gchar*, nalts + 1);
    paths[nalts] = NULL;

    for (i = 0; i < nalts; ++i) {
        g_return_if_fail (g_utf8_validate (alts[i], -1, NULL));
        music_notify ("appending alternative: %s", alts[i]);
        paths[i] = g_strdup (alts[i]);
    }

    playlist_append (listitem_new(g_strdup (path), paths, nalts));
}

void
playlist_fail ()
{
    PlaylistItem *item;
    guint nalts = 0;

    g_return_if_fail (playlist_current != NULL);

    item = LISTITEM (playlist_current);
    while (item->paths[nalts]) ++nalts;
    if (nalts - 1 > item->use_path) {
        ++item->use_path;
        music_notify ("track failed: '%s' trying: %s",
                      item->paths[item->use_path-1], PATH (item));
        /* try again */
        music_play ();
    } else {
        music_notify ("track failed: '%s'", PATH (item));
        playlist_remove (g_list_position (playlist, playlist_current));
    }
}

void
playlist_advance (gint num, gboolean loop)
{
    static GList *rand_cur = NULL;
    gboolean looped = FALSE, playing = music_playing;

    music_notify ("advancing %d track(s)", num);

    if (!playlist) return;
    
    if (main_random_order) {
        rand_cur = g_list_find (playlist_random, playlist_current->data);

        while (rand_cur && num > 0) {
            rand_cur = g_list_next (rand_cur);
            if (!rand_cur) {
                /* XXX re-randomize the whole thing */
                rand_cur = playlist_random;
                looped = TRUE;
            }
            --num;
        }
        while (rand_cur && num < 0) {
            rand_cur = g_list_previous (rand_cur);
            if (!rand_cur) {
                /* XXX re-randomize the whole thing */
                rand_cur = g_list_last (playlist_random);
                looped = TRUE;
            }
            ++num;
        }
        playlist_current = g_list_find (playlist, rand_cur->data);
    } else {
        while (playlist_current && num > 0) {
            playlist_current = g_list_next (playlist_current);
            if (!playlist_current) {
                playlist_current = playlist;
                looped = TRUE;
            }
            --num;
        }
        while (playlist_current && num < 0) {
            playlist_current = g_list_previous (playlist_current);
            if (!playlist_current) {
                playlist_current = g_list_last (playlist);
                looped = TRUE;
            }
            ++num;
        }
    }
    music_stop ();
    if ((!looped || loop) && playing)
        music_play ();
}

void
playlist_seek (gint num)
{
    GList *it = g_list_nth (playlist, num);

    if (it) {
        music_notify ("seeking to track %d", num);
        
        playlist_current = it;
        /* this function is used during load, and we don't want to start
           playing necessarily */
        if (main_status == QUARK_RUNNING) {
            music_stop ();
            music_play ();
        }
    }
}

void
playlist_clear ()
{
    music_notify ("clearing playlist");
    
    while (playlist) {
        listitem_free (playlist->data);
        playlist = g_list_delete_link (playlist, playlist);
    }
    playlist = playlist_current = NULL;
    g_list_free (playlist_random);
    playlist_random = NULL;
    music_stop ();
}

void
playlist_remove (gint num)
{
    GList *it = g_list_nth (playlist, num);
    gboolean playing = music_playing;

    if (it) {
        music_notify ("removing track %d", num);
        
        if (it == playlist_current) {
            music_stop ();
            playlist_advance (1, main_loop_at_end);
        }
        if (it == playlist_current)
            playlist_current = NULL;
        else if (playing)
            music_play ();

        listitem_free (it->data);
        playlist = g_list_delete_link (playlist, it);
    }
}

void
playlist_move (gint num, gint before)
{
    GList *it = g_list_nth (playlist, num);
    struct PlaylistItem *item;

    if (!it || num == before || num == before + 1) return;

    music_notify ("moving track %d before track %d", num, before);
    
    item = it->data;
    playlist = g_list_insert_before (playlist, g_list_nth (playlist, before),
                                     item);
    playlist = g_list_delete_link(playlist, it);
    /* XXX reinsert in the random order list? */
}

void
playlist_dump ()
{
    GList *it;
    gint i = 1;

    music_notify ("Current playlist:");

    for (it = playlist; it; it = g_list_next(it)) {
        gchar *base = g_path_get_basename (PATH (it->data));
        g_print ("%s %d %s\n",
                 (it->data == playlist_current->data ? ">" : " "), i++, base);
        g_free (base);
    }
}
