#include "config.h"

#include "gettext.h"
#include "editor.h"
#include <gtk/gtk.h>

enum {
    NUMBER_COLUMN,
    FILE_COLUMN,
    N_COLUMNS
};

enum {
    EDITOR_NUM_SIGNALS
};

#define QUARK_GCONF_ROOT "/apps/quark"
#define RECENT_DIR       QUARK_GCONF_ROOT "/recent_dir"

//static guint editor_signals[EDITOR_NUM_SIGNALS] = { 0 };
static GtkWidget *parent_class;

static void editor_class_init (EditorClass *klass);
static void editor_init (Editor *ed);
static void editor_finalize (GObject *object);

/* event handlers */
static void editor_open(GtkWidget *widget, gpointer data);
static void editor_save(GtkWidget *widget, gpointer data);
static void editor_clear(GtkWidget *widget, gpointer data);
static void editor_append(GtkWidget *widget, gpointer data);
static void editor_remove(GtkWidget *widget, gpointer data);
static void editor_close(GtkWidget *widget, gpointer data);
static void editor_ascend(GtkWidget *widget, gpointer data);
static void editor_descend(GtkWidget *widget, gpointer data);

GType
editor_get_type ()
{
    static GType type = 0;
    if (!type) { /* cache it */
        static const GTypeInfo info = {
            sizeof(EditorClass),
            NULL, /* base_init */
            NULL, /* base_finalize */
            (GClassInitFunc) editor_class_init,
            NULL, /* class_finalize */ 
            NULL, /* class_data */
            sizeof(Editor),
            0, /* n_preallocs */
            (GInstanceInitFunc) editor_init,
        };
        type = g_type_register_static (GTK_TYPE_WINDOW, "Editor", &info, 0);
    }
    return type;
}

static gboolean
editor_delete (GtkWidget *widget, GdkEventAny *event)
{
    gtk_widget_hide (widget);
    return TRUE;
}

static void
editor_class_init (EditorClass *klass)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);


    parent_class = g_type_class_peek_parent (klass);

    gobject_class->finalize = editor_finalize;

    widget_class->delete_event = editor_delete;
}

static void
editor_init (Editor *ed)
{
    GtkWidget *w, *m;
    GtkAccelGroup *accel;

    accel = gtk_accel_group_new ();

    ed->vbox = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (ed->vbox);
    gtk_container_add (GTK_CONTAINER (ed), ed->vbox);

    /* set up the menu */

    ed->menu = gtk_menu_bar_new();

    w = gtk_menu_item_new_with_mnemonic ("_File");
    gtk_menu_shell_append (GTK_MENU_SHELL (ed->menu), w);

    m = gtk_menu_new ();
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (w), m);

    w = gtk_image_menu_item_new_from_stock (GTK_STOCK_OPEN, accel);
    gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
    g_signal_connect (w, "activate", G_CALLBACK (editor_open), ed);
    w = gtk_image_menu_item_new_from_stock (GTK_STOCK_SAVE, accel);
    gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
    g_signal_connect (w, "activate", G_CALLBACK (editor_save), ed);
    w = gtk_separator_menu_item_new ();
    gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
    w = gtk_image_menu_item_new_from_stock (GTK_STOCK_CLOSE, accel);
    gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
    g_signal_connect (w, "activate", G_CALLBACK (editor_close), ed);

    w = gtk_menu_item_new_with_mnemonic ("_Edit");
    gtk_menu_shell_append (GTK_MENU_SHELL (ed->menu), w);

    m = gtk_menu_new ();
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (w), m);

    w = gtk_image_menu_item_new_from_stock (GTK_STOCK_ADD, accel);
    gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
    g_signal_connect (w, "activate", G_CALLBACK (editor_append), ed);
    w = gtk_image_menu_item_new_from_stock (GTK_STOCK_REMOVE, accel);
    gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
    g_signal_connect (w, "activate", G_CALLBACK (editor_remove), ed);
    w = gtk_separator_menu_item_new ();
    gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
    w = gtk_image_menu_item_new_from_stock (GTK_STOCK_CLEAR, accel);
    gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
    g_signal_connect (w, "activate", G_CALLBACK (editor_clear), ed);

    w = gtk_menu_item_new_with_mnemonic ("_Sort");
    gtk_menu_shell_append (GTK_MENU_SHELL (ed->menu), w);

    m = gtk_menu_new ();
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (w), m);

    w = gtk_image_menu_item_new_from_stock (GTK_STOCK_SORT_ASCENDING, accel);
    gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
    g_signal_connect (w, "activate", G_CALLBACK (editor_ascend), ed);
    w = gtk_image_menu_item_new_from_stock (GTK_STOCK_SORT_DESCENDING, accel);
    gtk_menu_shell_append (GTK_MENU_SHELL (m), w);
    g_signal_connect (w, "activate", G_CALLBACK (editor_descend), ed);
    w = gtk_separator_menu_item_new ();
    gtk_menu_shell_append (GTK_MENU_SHELL (m), w);

    gtk_widget_show_all (ed->menu);
    gtk_box_pack_start (GTK_BOX (ed->vbox), ed->menu, FALSE, FALSE, 0);

    /* set up the list */

    ed->data = gtk_list_store_new (N_COLUMNS,
                                   G_TYPE_INT,     /* song number */
                                   G_TYPE_STRING); /* song's filename */

    ed->list = gtk_tree_view_new_with_model (GTK_TREE_MODEL (ed->data));
    gtk_widget_show_all (ed->list);
    gtk_box_pack_start (GTK_BOX (ed->vbox), ed->list, TRUE, TRUE, 0);

    /* set up the buttons */

    ed->buttons = gtk_hbutton_box_new ();
    gtk_button_box_set_layout (GTK_BUTTON_BOX (ed->buttons),
                               GTK_BUTTONBOX_START);
    gtk_container_set_border_width (GTK_CONTAINER (ed->buttons), 5);
    gtk_box_set_spacing (GTK_BOX (ed->buttons), 5);
    gtk_widget_show (ed->buttons);

    ed->clear = gtk_button_new_from_stock (GTK_STOCK_CLEAR);
    gtk_widget_show (ed->clear);
    gtk_box_pack_start (GTK_BOX (ed->buttons), ed->clear, FALSE, FALSE, 0);
    g_signal_connect (ed->clear, "clicked", G_CALLBACK (editor_clear), ed);

    ed->append = gtk_button_new_from_stock (GTK_STOCK_ADD);
    gtk_widget_show (ed->append);
    gtk_box_pack_start (GTK_BOX (ed->buttons), ed->append, FALSE, FALSE, 0);
    g_signal_connect (ed->append, "clicked", G_CALLBACK (editor_append), ed);

    gtk_box_pack_end (GTK_BOX (ed->vbox), ed->buttons, FALSE, FALSE, 0);

    /* set up the whole window */
    gtk_window_set_title (GTK_WINDOW (ed), _("Edit Playlist - Quark"));
    gtk_window_set_role (GTK_WINDOW (ed), _("Playlist Editor"));
    gtk_window_set_default_size (GTK_WINDOW (ed), 300, 300);
    gtk_window_add_accel_group (GTK_WINDOW (ed), accel);
}

static void
editor_finalize (GObject *object)
{
}

GtkWidget *
editor_new (QuarkClient *qc, GConfClient *gconf, GdkPixbuf *icon)
{
    Editor *ed;

    g_return_val_if_fail (qc != NULL, NULL);
    g_return_val_if_fail (gconf != NULL, NULL);

    ed = g_object_new (EDITOR_TYPE, NULL);
    ed->qc = qc;
    ed->gconf = gconf;
    ed->icon = icon;

    if (icon)
        gtk_window_set_icon (GTK_WINDOW (ed), icon);

    return GTK_WIDGET (ed);
}

static void
editor_open(GtkWidget *widget, gpointer data)
{
}

static void
editor_save(GtkWidget *widget, gpointer data)
{
}

static void
editor_clear(GtkWidget *widget, gpointer data)
{
    editor_clear_files (EDITOR (data));
}

static void
editor_append(GtkWidget *widget, gpointer data)
{
    editor_add_files (EDITOR (data), TRUE);
}

static void
editor_remove(GtkWidget *widget, gpointer data)
{
}

static void
editor_close(GtkWidget *widget, gpointer data)
{
    editor_delete (data, NULL);
}

static void
editor_ascend(GtkWidget *widget, gpointer data)
{
}

static void
editor_descend(GtkWidget *widget, gpointer data)
{
}

void
editor_add_files (Editor *ed, gboolean append)
{
    GtkWidget *sel;
    gboolean r = TRUE;
    gchar *p, *p2;

    g_return_if_fail (ed != NULL);

    sel = gtk_file_selection_new (append ?
                                  _("Append Songs - Quark") :
                                  _("Play Songs - Quark"));
    gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (sel), TRUE);
    if (ed->icon)
        gtk_window_set_icon (GTK_WINDOW (sel), ed->icon);

    if ((p = gconf_client_get_string (ed->gconf, RECENT_DIR, NULL)))
        gtk_file_selection_set_filename (GTK_FILE_SELECTION (sel), p);

    if (gtk_dialog_run (GTK_DIALOG (sel)) == GTK_RESPONSE_OK) {
        gchar **files;

        files = gtk_file_selection_get_selections (GTK_FILE_SELECTION (sel));
        if (!append)
            r = quarkclient_clear (ed->qc, NULL);
        if (r) r = quarkclient_add_tracks (ed->qc, files, NULL);
        if (r) quarkclient_play (ed->qc, NULL);

        p = g_path_get_dirname (files[0]);
        p2 = g_strconcat (p, G_DIR_SEPARATOR_S, NULL); /* make it into a dir */
        gconf_client_set_string (ed->gconf, RECENT_DIR, p2, NULL);
        g_free (p2);
        g_free (p);

        g_strfreev (files);
    }    

    gtk_widget_destroy (sel);
}

void
editor_clear_files (Editor *ed)
{
    quarkclient_clear (ed->qc, NULL);
}
